"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RabbitmqConsumer = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const amqplib_1 = tslib_1.__importDefault(require("amqplib"));
const debug_1 = tslib_1.__importDefault(require("debug"));
const events_1 = require("events");
const index_1 = require("./index");
const debug = debug_1.default('loopback:rabbitmq:consumer');
let RabbitmqConsumer = class RabbitmqConsumer extends events_1.EventEmitter {
    constructor(componentConfig) {
        super();
        this.componentConfig = componentConfig;
        this.componentConfig = { ...index_1.ConfigDefaults, ...this.componentConfig };
        debug('criou uma instancia RabbitmqServer');
        const { retries, interval } = this.componentConfig.consumer;
        this.retry = 0;
        this.retries = retries;
        this.interval = interval;
    }
    async getConnection() {
        if (this.connection) {
            return this.connection;
        }
        this.connection = await amqplib_1.default.connect(this.componentConfig.options);
        debug('getConnection::connection created');
        if (this.retries === 0 || this.retries > this.retry + 1) {
            debug('connection:retries', this.retries);
            const restart = (err) => {
                debug('Connection error occurred.', err);
                if (this.connection) {
                    this.connection.removeListener('error', restart);
                    this.connection = undefined;
                    this.channel = undefined;
                }
                this.timeout(this.interval);
            };
            const onClose = () => {
                if (this.connection)
                    this.connection.removeListener('close', onClose);
                restart(new Error('Connection closed by remote host'));
            };
            this.connection.removeAllListeners('error');
            this.connection.removeAllListeners('close');
            this.connection.on('error', restart);
            this.connection.on('close', onClose);
        }
        return this.connection;
    }
    timeout(ms) {
        debug('timeout::started');
        if (this.timeoutId) {
            clearTimeout(this.timeoutId);
        }
        const promise = new Promise((resolve, reject) => {
            this.timeoutId = setTimeout(() => {
                debug(`timeout::Connection retry ${this.retry + 1} in ${ms} ms`);
                if (!this.connection) {
                    ++this.retry;
                    this.getConnection().then(() => {
                        this.retry = 0;
                        debug('timeout::connection created');
                        if (!this.channel) {
                            this.getChannel().then(() => {
                                debug('timeout::channel created');
                                this.emit('re-established-connection');
                                resolve();
                            }, () => { });
                        }
                        else {
                            this.emit('re-established-connection');
                            resolve();
                        }
                    }, (error) => {
                        this.timeout(this.interval);
                        debug(error.message);
                        reject(error);
                    });
                }
            }, ms);
        });
        const onResolve = () => { };
        const onReject = () => { };
        promise.then(onResolve, onReject);
    }
    async getChannel() {
        const connection = await this.getConnection();
        if (!this.channel) {
            this.channel = await connection.createChannel();
            debug('getChannel::channel created');
            await this.setupInitChannel(this.channel);
            debug('getChannel::setupInitChannel called');
        }
        return this.channel;
    }
    async setupInitChannel(channel) {
        var _a, _b, _c;
        for (const exchange of this.componentConfig.exchanges) {
            const { exchange: createdExchange, } = await channel.assertExchange(exchange.name, (_a = exchange.type) !== null && _a !== void 0 ? _a : this.componentConfig.defaultExchangeType, { ...this.componentConfig.defaultExchangeOptions, ...exchange.options });
            const queues = (_b = exchange.queues) !== null && _b !== void 0 ? _b : [];
            for (const q of queues) {
                const { queue } = await channel.assertQueue((_c = q.queue) !== null && _c !== void 0 ? _c : '', {
                    ...this.componentConfig.defaultQueueOptions,
                    ...q.queueOptions,
                });
                const routingKeys = Array.isArray(q.routingKey)
                    ? q.routingKey
                    : [q.routingKey];
                await Promise.all(routingKeys.map(route => {
                    debug('bindQueue: ', `${queue} => ${createdExchange} => ${route}`);
                    return channel.bindQueue(queue, createdExchange, route);
                }));
            }
        }
        await channel.prefetch(this.componentConfig.prefetchCount);
    }
    async setupSubscriberChannel(handler, msgOptions) {
        var _a;
        const channel = await this.getChannel();
        const { exchange, routingKey, allowNonJsonMessages } = msgOptions;
        const { queue } = await channel.assertQueue((_a = msgOptions.queue) !== null && _a !== void 0 ? _a : '', {
            ...this.componentConfig.defaultQueueOptions,
            ...msgOptions.queueOptions,
        });
        const routingKeys = Array.isArray(routingKey) ? routingKey : [routingKey];
        await Promise.all(routingKeys.map(route => {
            debug('bindQueue: ', `${queue} => ${exchange} => ${route}`);
            return channel.bindQueue(queue, exchange, route);
        }));
        // eslint-disable-next-line @typescript-eslint/no-misused-promises
        await channel.consume(queue, async (message) => {
            var _a;
            debug(`queue:${queue}: new Message`);
            try {
                if (message == null) {
                    throw new Error('Received null message');
                }
                const response = await this.handleMessage(handler, message, allowNonJsonMessages);
                if (response instanceof index_1.Nack) {
                    channel.nack(message, false, response.requeue);
                    return;
                }
                if (response) {
                    throw new Error('Received response from consumer handler. Consumer handlers should only return void');
                }
                channel.ack(message);
            }
            catch (error) {
                debug('error: ', error, message);
                if (message === null) {
                    return;
                }
                else {
                    const errorHandler = (_a = msgOptions.errorHandler) !== null && _a !== void 0 ? _a : index_1.getHandlerErrorBehavior(this.componentConfig.defaultConsumerErrorBehavior);
                    await errorHandler(channel, message, error);
                }
            }
        });
        const methodName = handler.name.replace('bound ', '');
        debug('registered:consumer: ', queue);
        debug('registered:consumer:method: ', methodName);
    }
    handleMessage(handler, msg, allowNonJsonMessages) {
        let message = undefined;
        if (msg.content) {
            if (allowNonJsonMessages) {
                try {
                    message = JSON.parse(msg.content.toString());
                }
                catch (_a) {
                    // Let handler handle parsing error, it has the raw message anyway
                    message = undefined;
                }
            }
            else {
                message = JSON.parse(msg.content.toString());
            }
        }
        const methodName = handler.name.replace('bound ', '');
        debug(`consumer:${methodName} new message: %o`, message);
        debug('registered:consumer:method:call ', methodName);
        return handler(message, msg);
    }
};
RabbitmqConsumer = tslib_1.__decorate([
    core_1.bind({ scope: core_1.BindingScope.SINGLETON }),
    tslib_1.__param(0, core_1.config({ fromBinding: index_1.RabbitmqBindings.COMPONENT })),
    tslib_1.__metadata("design:paramtypes", [Object])
], RabbitmqConsumer);
exports.RabbitmqConsumer = RabbitmqConsumer;
//# sourceMappingURL=rabbitmq.consumer.js.map