"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RabbitmqProducer = exports.isObject = exports.jsonReplacer = void 0;
const tslib_1 = require("tslib");
/* eslint-disable @typescript-eslint/no-explicit-any */
const core_1 = require("@loopback/core");
const amqplib_1 = tslib_1.__importDefault(require("amqplib"));
const debug_1 = tslib_1.__importDefault(require("debug"));
const index_1 = require("./index");
const debug = debug_1.default('loopback:rabbitmq:producer');
const jsonReplacer = (key, value) => typeof value === 'undefined' ? null : value;
exports.jsonReplacer = jsonReplacer;
const isObject = (obj) => {
    const type = typeof obj;
    return type === 'function' || (type === 'object' && !!obj);
};
exports.isObject = isObject;
let RabbitmqProducer = class RabbitmqProducer {
    constructor(componentConfig = index_1.ConfigDefaults) {
        this.componentConfig = componentConfig;
        this.channelIsClosing = false;
        this.connectionIsClosing = false;
        this.componentConfig = { ...index_1.ConfigDefaults, ...this.componentConfig };
        debug('created an instance of RabbitmqProducer: %o', this.componentConfig);
    }
    async getConnection() {
        var _a, _b;
        if ((_a = this.componentConfig.producer) === null || _a === void 0 ? void 0 : _a.idleTimeoutMillis)
            this.timeout((_b = this.componentConfig.producer) === null || _b === void 0 ? void 0 : _b.idleTimeoutMillis);
        if (!this.connection || this.connectionIsClosing) {
            this.connection = await amqplib_1.default.connect(this.componentConfig.options);
            this.connectionIsClosing = false;
            debug('getConnection::connection created');
            const restart = (err) => {
                if (this.connection)
                    this.connection.removeListener('error', restart);
                this.connection = undefined;
                this.channel = undefined;
            };
            const onClose = () => {
                if (this.connection)
                    this.connection.removeListener('close', onClose);
                this.connectionIsClosing = true;
                restart(new Error('Connection closed by remote host'));
            };
            this.connection.removeAllListeners('error');
            this.connection.removeAllListeners('close');
            this.connection.on('error', restart);
            this.connection.on('close', onClose);
        }
        return this.connection;
    }
    async getChannel() {
        const connection = await this.getConnection();
        if (!this.channel || this.channelIsClosing) {
            this.channel = await connection.createChannel();
            this.channelIsClosing = false;
            debug('getChannel::channel created');
            const restart = (err) => {
                if (this.channel)
                    this.channel.removeAllListeners('error');
                this.channel = undefined;
            };
            const onClose = () => {
                if (this.channel)
                    this.channel.removeAllListeners('close');
                this.channelIsClosing = true;
                restart(new Error('Connection closed by remote host'));
            };
            this.channel.removeAllListeners('error');
            this.channel.removeAllListeners('close');
            this.channel.on('error', restart);
            this.channel.on('close', onClose);
        }
        return this.channel;
    }
    timeout(ms) {
        if (this.timeoutId) {
            debug(`timeout::Delayed for ${ms} ms`);
            clearTimeout(this.timeoutId);
        }
        const promise = new Promise(resolve => {
            this.timeoutId = setTimeout(() => {
                debug('timeout::Ending producer due to timeout');
                if (this.channel) {
                    this.channelIsClosing = true;
                    this.channel.close().then(() => {
                        debug('timeout::channel closed');
                        this.channel = undefined;
                        if (this.connection) {
                            this.connection.close().then(() => {
                                debug('timeout::connection closed');
                                this.connection = undefined;
                                resolve();
                            }, () => { });
                        }
                        else {
                            resolve();
                        }
                    }, () => { });
                }
                if (this.connection) {
                    this.connectionIsClosing = true;
                    this.connection.close().then(() => {
                        debug('timeout::connection closed');
                        this.connection = undefined;
                        resolve();
                    }, () => { });
                }
                else {
                    resolve();
                }
            }, ms);
        });
        const onResolve = () => { };
        const onReject = () => { };
        promise.then(onResolve, onReject);
    }
    parseToBuffer(message) {
        let buffer;
        if (message instanceof Buffer) {
            buffer = message;
        }
        else if (message instanceof Uint8Array) {
            buffer = Buffer.from(message);
        }
        else if (message != null) {
            buffer = Buffer.from(JSON.stringify(message /*, jsonReplacer */));
        }
        else {
            buffer = Buffer.alloc(0);
        }
        return buffer;
    }
    async produce(queue, content, durable = true, persistent = true) {
        const buffer = this.parseToBuffer(content);
        const channel = await this.getChannel();
        //Cria uma Queue Caso não exista
        await channel.assertQueue(queue, { durable });
        channel.sendToQueue(queue, buffer, { persistent });
    }
    async publish(exchange, routingKey, message, options) {
        const channel = await this.getChannel();
        const buffer = this.parseToBuffer(message);
        const routingKeys = Array.isArray(routingKey) ? routingKey : [routingKey];
        for (const route of routingKeys) {
            channel.publish(exchange, route, buffer, options);
        }
    }
};
RabbitmqProducer = tslib_1.__decorate([
    core_1.bind({ scope: core_1.BindingScope.SINGLETON }),
    tslib_1.__param(0, core_1.config({ fromBinding: index_1.RabbitmqBindings.COMPONENT })),
    tslib_1.__metadata("design:paramtypes", [Object])
], RabbitmqProducer);
exports.RabbitmqProducer = RabbitmqProducer;
//# sourceMappingURL=rabbitmq.producer.js.map